/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

namespace IRDB_SDK
{
	using namespace std;
	enum CFGEdgeTypeEnum { cetFallthroughEdge, cetTargetEdge, cetIndirectEdge };
	using CFGEdgeType_t = set<CFGEdgeTypeEnum>;

	class ControlFlowGraph_t
	{
		protected:
			ControlFlowGraph_t() { }
			ControlFlowGraph_t(const ControlFlowGraph_t& copy)  = delete;
		public:

			virtual BasicBlock_t*          getEntry()                                                        const = 0 ;
			virtual Function_t*            getFunction()                                                     const = 0 ;
			virtual const BasicBlockSet_t& getBlocks()                                                       const = 0 ;
			virtual bool                   hasEdge(BasicBlock_t *p_src, BasicBlock_t *p_tgt)                 const = 0 ;
			virtual CFGEdgeType_t          getEdgeType(const BasicBlock_t *p_src, const BasicBlock_t *p_tgt) const = 0 ;
			virtual void                   dump(ostream &os=cout)                                            const = 0 ;

		// factories
			static unique_ptr<ControlFlowGraph_t> factory(Function_t* func);

	};
	ostream& operator<<(ostream& os, const ControlFlowGraph_t& cfg);

}
