/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

#include <irdb-core>
#include <irdb-cfg>
#include <set>

namespace IRDB_SDK
{
	using namespace std;

	class Loop_t; // forward decl
	
	using LoopSet_t = set<Loop_t*>;

	class Loop_t
	{
		protected: 
			Loop_t() {}
			Loop_t(const Loop_t& copy) = delete;

		public:
			virtual ~Loop_t() {} 

			virtual BasicBlock_t*   getPreheader()   const = 0;
			virtual BasicBlock_t*   getHeader()      const = 0;
			virtual BasicBlockSet_t getAllBlocks()   const = 0;
			virtual BasicBlockSet_t getOuterBlocks() const = 0;
			virtual LoopSet_t       getInnerLoops()  const = 0;

			virtual bool isBlockInLoop       (const BasicBlock_t* blk)  const = 0;
			virtual bool isBlockInInnerLoop  (const BasicBlock_t* blk)  const = 0;
			virtual bool isBlockOuterLoopOnly(const BasicBlock_t* blk)  const = 0;
	};

	class LoopNest_t
	{
		protected: 
			LoopNest_t() {}
			LoopNest_t(const LoopNest_t& copy) = delete;

		public:
			virtual ~LoopNest_t() {}

			virtual LoopSet_t           getAllLoops()   const = 0;
			virtual LoopSet_t           getOuterLoops() const = 0;
			virtual Function_t*         getFunction()   const = 0;
			virtual ControlFlowGraph_t* getCFG()       const = 0;
	};
}
