/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

#include <sstream>

namespace IRDB_SDK
{

	using namespace std;

	template <class T>
	inline string to_hex_string (const T& t)
	{
		stringstream ss;
		ss << hex << t;
		return ss.str();
	}

	 

	template<class T> inline T strtoint(const string& s)
	{
		stringstream str(s);
		auto off=T();

		str >> off;

		return off;
	}

	// nb: relies on srand() being previously set 
	template <typename IterType, typename Funct>
	inline Funct for_randomOrder_each(const IterType &b, const IterType & e, const Funct &callback)
	{
		auto m=map<int,const typename iterator_traits<IterType>::value_type *>();
		for_each(b,e, [&](const typename iterator_traits<IterType>::value_type & o)
		{
		       while(true)
		       {
			       auto rn=rand();
			       if(m.find(rn)==m.end())
			       {
				       m[rn]=&o;
				       break;
			       }
		       }
		});
		for_each(m.begin(), m.end(), [&](const pair<int,const typename iterator_traits<IterType>::value_type *> &p)
		{
		       callback(*p.second);
		});
		return callback;
	}



}


